package com.jiuqi.sql;

import java.util.Date;

public abstract class InputDataSet {
   private boolean active;
   private FieldInfoList fieldInfos = new FieldInfoList();
   private InputRecordBuffer buffer;
   private DataSetListener listener = null;

   public void resetListener(DataSetListener listerner) {
      this.listener = listerner;
   }

   private FieldInfo findField(String fieldName) throws DBException {
      int index = this.fieldInfos.find(fieldName);
      if (index < 0) {
         throw new DBException(fieldName + " Not Exists");
      } else {
         FieldInfo result = this.fieldInfos.get(index);
         return result;
      }
   }

   public abstract InputDataSetMetaData getMetaData() throws DBException;

   protected abstract boolean getNextRecord(InputRecordBuffer var1) throws DBException;

   protected abstract void initFields(FieldInfoList var1) throws DBException;

   protected abstract InputRecordBuffer internalOpen() throws DBException;

   protected abstract void internalClose() throws DBException;

   protected InputRecordBuffer recordBuffer() {
      return this.buffer;
   }

   public void open() throws DBException {
      if (!this.isActive()) {
         this.initFields(this.fieldInfos);
         this.buffer = this.internalOpen();
         this.fieldInfos.refreshFinder();
         this.active = true;
         if (this.listener != null) {
            this.listener.afterEvent(new DataSetEvent(this, 1));
         }
      }

   }

   public void first() throws DBException {
      throw new DBException("尚未实现！");
   }

   public boolean isActive() {
      return this.active;
   }

   public boolean next() throws DBException {
      boolean result = false;

      do {
         result = this.getNextRecord(this.buffer);
      } while(result && this.buffer.isDeleted());

      return result;
   }

   public void close() throws DBException {
      if (this.isActive()) {
         this.internalClose();
         this.active = false;
         if (this.listener != null) {
            this.listener.afterEvent(new DataSetEvent(this, 2));
         }
      }

   }

   public int fieldIndexOf(String fieldName) {
      int result = this.fieldInfos.find(fieldName);
      return result < 0 ? result : result + 1;
   }

   public boolean getBoolean(int index) throws DBException {
      return this.buffer.readBoolean(this.fieldInfos.get(index - 1));
   }

   public boolean getBoolean(String fieldName) throws DBException {
      return this.buffer.readBoolean(this.findField(fieldName));
   }

   public int getInt(int index) throws DBException {
      return this.buffer.readInt(this.fieldInfos.get(index - 1));
   }

   public int getInt(String fieldName) throws DBException {
      return this.buffer.readInt(this.findField(fieldName));
   }

   public long getLong(int index) throws DBException {
      return this.buffer.readLong(this.fieldInfos.get(index - 1));
   }

   public long getLong(String fieldName) throws DBException {
      return this.buffer.readLong(this.findField(fieldName));
   }

   public String getString(int index) throws DBException {
      return this.buffer.readString(this.fieldInfos.get(index - 1));
   }

   public String getString(String fieldName) throws DBException {
      return this.buffer.readString(this.findField(fieldName));
   }

   public double getDouble(int index) throws DBException {
      return this.buffer.readDouble(this.fieldInfos.get(index - 1));
   }

   public double getDouble(String fieldName) throws DBException {
      return this.buffer.readDouble(this.findField(fieldName));
   }

   /** @deprecated */
   public Date readDate(int index) throws DBException {
      return this.buffer.readDate(this.fieldInfos.get(index - 1));
   }

   /** @deprecated */
   public Date readDate(String fieldName) throws DBException {
      return this.buffer.readDate(this.findField(fieldName));
   }

   public Date getDate(int index) throws DBException {
      return this.buffer.readDate(this.fieldInfos.get(index - 1));
   }

   public Date getDate(String fieldName) throws DBException {
      return this.buffer.readDate(this.findField(fieldName));
   }

   private Object getObject(FieldInfo fieldInfo) throws DBException {
      switch (fieldInfo.type) {
         case 1:
            boolean value = this.buffer.readBoolean(fieldInfo);
            return this.buffer.wasNull() ? null : new Boolean(value);
         case 2:
            Date value = this.buffer.readDate(fieldInfo);
            return this.buffer.wasNull() ? null : value;
         case 3:
            double value = this.buffer.readDouble(fieldInfo);
            return this.buffer.wasNull() ? null : new Double(value);
         case 4:
         default:
            return null;
         case 5:
            int value = this.buffer.readInt(fieldInfo);
            return this.buffer.wasNull() ? null : new Integer(value);
         case 6:
            String value = this.buffer.readString(fieldInfo);
            return this.buffer.wasNull() ? null : value;
         case 7:
         case 8:
            long value = this.buffer.readLong(fieldInfo);
            return this.buffer.wasNull() ? null : new Long(value);
         case 9:
            return null;
      }
   }

   public final Object getObject(int index) throws DBException {
      return this.getObject(this.fieldInfos.get(index - 1));
   }

   public final Object getObject(String fieldName) throws DBException {
      return this.getObject(this.findField(fieldName));
   }

   public boolean wasNull() throws DBException {
      return this.buffer.wasNull();
   }
}
